/*
 * @author Walther Neuper, member of the ISAC-team, 
 * Copyright (c) 2005 by Walther Neuper
 * created Jun 18, 2005 2:56:04 PM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.bridge;

import isac.kestore.KEStoreServices;
import isac.util.CalcChangedEvent;
import isac.util.Formalization;
import isac.util.formulae.CalcHead;
import isac.util.formulae.Match;
import isac.util.formulae.ProblemID;
import isac.util.interfaces.ICalcIterator;
import isac.util.interfaces.IToCalc;
import isac.util.interfaces.IToUser;
import isac.util.parser.FormalizationDigest;
import isac.util.tactics.SimpleTactic;
import isac.util.tactics.StringListTactic;
import isac.util.tactics.Tactic;

import java.util.Vector;

import junit.framework.TestCase;

/**
 * @author Walther Neuper Sep 28, 2004, 3:37:02 PM
 * @version 1.0
 * @see isac.functest.specify.TestSpecify for related tests
 */
public class TestSpecify extends TestCase {

    Formalization fmz_;

    MathEngine me_;

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
      //load the file of the example,
      //see TestKEStoreServices#testLoadContentMinisubpbl
      KEStoreServices ks = new KEStoreServices("isac/xmldata");
      Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_7a.xml");
      //parse out the Formalization
      //see TestFormalizationDigest#testParseExplMini
      String xmlstr = (String) vec.firstElement();
      FormalizationDigest fd = new FormalizationDigest();
      fmz_ = fd.parseFormalization((String) xmlstr);

      MathEngine.init("localhost");
      me_ = MathEngine.getMathEngine();
    }

    /**
     * {\bf\UC{Having \isac{} Complete the
     * Specification}\label{SPECIFY:COMPLETE:spec_}\\} this is done with the
     * same button as 'complete CalcHead'
     * 
     * @see isac.bridge.TestSpecify#testCompleteSpecification() for how this
     *      looks like on the fron-end
     */
    public void testCompleteSpecification() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestSpecify#testCompleteSpecification");
 
        //start the calculation
        IToCalc ct = me_.startCalculation(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model ...
        ct.autoCalculate(IToCalc.SCOPE_MODEL, 0);//FIXXXME.WN0504 completeModel
        System.out.println(".end step_b-----");
        //end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Specification by completeCalcHead
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getSpecification().toSMLString(),
                "\"e_domID\",[\"e_pblID\"],[\"e_metID\"]");
        ct.completeCalcHead();
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getSpecification().toSMLString(), "\"DiffApp.thy\","
                + "[\"maximum_of\",\"function\"],"
                + "[\"DiffApp\",\"max_by_calculus\"]");
        System.out
                .println("---END isac.bridge.TestSpecify#testCompleteSpecification");
    }

    /**
     * {\bf\UC{Having \isac{} Complete One Field of the
     * Specification}\label{SPECIFY:COMPLETE:specone}\\}
     * 
     * @see isac.bridge.TestSpecify#testCompleteOneFieldOfSpecification() for
     *      how this looks like on the fron-end
     */
    public void testCompleteOneFieldOfSpecification() throws Exception {
        System.out
                .println("---BEGIN isac.bridge.TestSpecify#testCompleteOneFieldOfSpecification");


        //start the calculation
        IToCalc ct = me_.startCalculation(fmz_);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        ICalcIterator ci = ct.getActiveFormula();
        CalcHead ch = null;
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Model ...
        ct.autoCalculate(IToCalc.SCOPE_MODEL, 0);
        System.out.println(".end step_b-----");
        //end step_b ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the Specification
        //could be continued by autoCalculate, i.e. by <Next>-button,
        //but here by stepwise input
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getSpecification().toSMLString(),
                "\"e_domID\",[\"e_pblID\"],[\"e_metID\"]");
        //input the Theory ...
        Tactic tac = new SimpleTactic("Specify_Theory", "DiffApp.thy");
        ct.setNextTactic(tac);
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getSpecification().toSMLString(),
                "\"DiffApp.thy\",[\"e_pblID\"],[\"e_metID\"]");
        //input the Problem ...
        Vector keylist = new Vector();
        keylist.add("maximum_of");
        keylist.add("function");
        tac = new StringListTactic("Specify_Problem", keylist);
        ct.setNextTactic(tac);
        //input the Method ...
        keylist = new Vector();
        keylist.add("DiffApp");
        keylist.add("max_by_calculus");
        tac = new StringListTactic("Specify_Method", keylist);
        ct.setNextTactic(tac);
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getSpecification().toSMLString(), "\"DiffApp.thy\","
                + "[\"maximum_of\",\"function\"],"
                + "[\"DiffApp\",\"max_by_calculus\"]");
        System.out
                .println("---END isac.bridge.TestSpecify#testCompleteOneFieldOfSpecification");
    }

    /**
     * {\bf\UC{Having \isac{} Refine the Problem
     * Automatically}\label{SPECIFY:refine-auto}\\} test match and refine with
     * x^^^2 + 4*x + 5 = 2 sml/systest/FE-interface.sml --- tryMatchProblem,
     * tryRefineProblem ---
     * 
     * @see isac.functest.problems.TestRefinement#testAutomaticallyRefinement()
     *      for how this looks like at the front-end.
     * 
     * this run requires another example !!!
     */
    public void testMatchRefine() throws Exception {
        System.out.println("---BEGIN isac.bridge.TestModel#testMatchRefine");

        //load the file of the example,
        //see TestKEStoreServices#testLoadContentMinisubpbl
        KEStoreServices ks = new KEStoreServices("isac/xmldata");
        Vector vec = ks.loadContent("exp", "exp_IsacCore_Tests_1d.xml");
        //parse out the Formalization
        //see TestFormalizationDigest#testParseExplMini
        String xmlstr = (String) vec.firstElement();
        FormalizationDigest fd = new FormalizationDigest();
        Formalization fmz = fd.parseFormalization((String) xmlstr);
        System.out.println(".end step_a-----");
        //end step_a ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //start the calculation
        MathEngine.init("localhost");
        MathEngine me = MathEngine.getMathEngine();

        IToCalc ct = me.startCalculation(fmz);
        IToUser user = new MockIToUser();
        ct.addDataChangeListener(user);
        CalcChangedEvent cce = null;
        System.out.println(".end step_b-----");
        //end step_b
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //fill the CalcHead with Descriptions by 2-times <NEXT>:
        //tac = Model_Problem ...
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        //and one field: tac = Add_Given "equality (x ^^^ 2 + 4 * x + 5 = 2)"
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ICalcIterator ci = ct.getActiveFormula();
        //end step_c
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        //trymatch ... go into the problem-hierarchy with this CalcHeads Model
        //this Match will show some incomplete items
        ProblemID id = new ProblemID();
        id.addString("univariate");
        id.addString("equation");
        Match m = ci.tryMatchProblem(id);
        assertEquals(m.getHeadline().toSMLString(), "??.empty");//...TODO.WN050626
        assertEquals(m.getKEStoreID().toSMLString(),
                "[\"univariate\",\"equation\"]");
        assertEquals(m.getModel().getGiven().toString(),
                "[\"incomplete solveFor\", "
                        + "\"correct equality (x ^^^ 2 + 4 * x + 5 = 2)\"]");
        assertEquals(m.getModel().getWhere().toString(),
                "[\"correct matches (?a = ?b) (x ^^^ 2 + 4 * x + 5 = 2)\"]");
        assertEquals(m.getModel().getFind().toString(),
                "[\"incomplete solutions []\"]");
        assertEquals(m.getModel().getRelate().toString(), "[]");
        assertEquals(m.getStatus(), "incorrect");
        //end step_d
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //complete the CalcHeads Model in the Worksheet by 2-times <NEXT>:
        //tac = Add_Given "solveFor x"
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        //tac = Add_Find "solutions L"
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        //end step_e
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        //trymatch ... go into the problem-hierarchy with this CalcHeads Model
        //this Match will show all items complete:
        m = ci.tryMatchProblem(id);
        assertEquals(m.getHeadline().toSMLString(), "??.empty");//...TODO.WN050626
        assertEquals(m.getModel().getGiven().toString(),
                "[\"correct equality (x ^^^ 2 + 4 * x + 5 = 2)\", "
                        + "\"correct solveFor x\"]");
        assertEquals(m.getModel().getFind().toString(),
                "[\"correct solutions L\"]");
        assertEquals(m.getStatus(), "incorrect");//the specification is
        // missing! TODO.WN0506 correct Match in SML !!!

        //this _NOT_ a matching problem:
        id = new ProblemID();
        id.addString("linear");
        id.addString("univariate");
        id.addString("equation");
        m = ci.tryMatchProblem(id);
        assertEquals(
                m.getModel().getWhere().toString(),
                "[\"false False\", "//WN050626 this seems to be for SMLtesting
                        + "\"correct ~ lhs (x ^^^ 2 + 4 * x + 5 = 2) is_polyrat_in x\", "
                        + "\"correct ~ rhs (x ^^^ 2 + 4 * x + 5 = 2) is_polyrat_in x\", "
                        + "\"false lhs (x ^^^ 2 + 4 * x + 5 = 2) has_degree_in x = 1\", "
                        + "\"false rhs (x ^^^ 2 + 4 * x + 5 = 2) has_degree_in x = 1\"]");
        //end step_f
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //search for a matching problem in the problem-hierarchy:
        //this search _FAILES_ because refine does _NOT_ return a new ProblemID
        m = ci.tryRefineProblem(id);
        assertEquals(m.getKEStoreID().toSMLString(),
                "[\"linear\",\"univariate\",\"equation\"]");
        //this search is _SUCCESSFUL_ because it returns a _NEW_ ProblemID
        //(and in this case the Model will _ALWAYS_ be "correct"!)
        id = new ProblemID();
        id.addString("univariate");
        id.addString("equation");
        m = ci.tryRefineProblem(id);
        assertEquals(m.getKEStoreID().toSMLString(),
                "[\"normalize\",\"polynomial\",\"univariate\",\"equation\"]");
        //end step_g
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

        //transfer the problem found back to the Worksheet
        Vector keylist = new Vector();
        keylist.add("normalize");
        keylist.add("polynomial");
        keylist.add("univariate");
        keylist.add("equation");
        Tactic tac = new StringListTactic("Specify_Problem", keylist);
        ct.setNextTactic(tac);
        //ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        //TODO.WN0506 ... still implicit in setNextTactic
        CalcHead ch = (CalcHead) ci.getElement();
        //FIXXME.WN050426 in the CalcHead above the Problem should be set !!!
        //and the calchead_status_ should be still "incomplete" due to
        // ..incomplete specification (theory, method missing)
        //after inserting Theory and Method the CalcHead is complete:
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ct.autoCalculate(IToCalc.SCOPE_CALCULATION, 1);
        ch = (CalcHead) ci.getElement();
        assertEquals(ch.getStatusString(), "correct");

        System.out.println("---END isac.bridge.TestModel#testMatchRefine");
    }

}