/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.browser.problem;

import isac.gui.Worksheet;
import isac.gui.browser.ContentPanel;
import isac.gui.browser.TabPanel;
import isac.gui.util.Hierarchy;
import isac.gui.util.ProblemNode;
import isac.util.formulae.KEStoreID;

import java.awt.event.*;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;

/**
 * @author Mario Hochreiter
 * @version 0.1
 *  
 */
public class ProblemBrowserPanel extends TabPanel implements ActionListener {

    static Logger logger = Logger
            .getLogger(ProblemBrowserPanel.class.getName());

    protected JPopupMenu popup_menu_;

    protected JMenuItem try_refine_;

    protected JMenuItem try_match_;

    protected ProblemNode sel_node_;

    protected TreePath sel_path_;

    public ProblemBrowserPanel(JFrame root, Hierarchy hierarchy) {
        super(root, hierarchy);
        this.buildMenu();
        this.initMouseListener();
    }

    protected void buildMenu() {
        popup_menu_ = new JPopupMenu();
        ResourceBundle rb = ResourceBundle.getBundle("ProblemBrowserPanel",
                root_.getLocale());
        try_refine_ = new JMenuItem(rb.getString("refineMenuItem"));
        try_match_ = new JMenuItem(rb.getString("matchMenuItem"));
        try_refine_.addActionListener(this);
        try_match_.addActionListener(this);
        popup_menu_.add(try_refine_);
        popup_menu_.add(try_match_);

    }

    protected void initMouseListener() {
        ml_ = new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                int selRow = view_tree_.getRowForLocation(e.getX(), e.getY());
                sel_path_ = view_tree_.getPathForLocation(e.getX(), e.getY());
                if (selRow != -1 && selRow != 0) {
                    sel_node_ = (ProblemNode) sel_path_.getLastPathComponent();
                    if ((e.getClickCount() == 1)
                            && (MouseEvent.BUTTON1 == e.getButton())
                            && isTryMatchAllowed()) {
                        logger.debug("clickCount: 1");
                        KEStoreID chID = createCalcHeadCompoundIDFromPath();
                        doTryMatch(chID);
                        // do something
                    } else if (e.getClickCount() == 2) {
                        ContentPanel cp = root_.getContentPanel();
                        if (sel_node_.getContentref() != null) {
                            cp.setContent("pbl", sel_node_.getContentref());
                        }
                        root_.showContentFrame();
                    } else if (e.getButton() == MouseEvent.BUTTON3) {
                        Worksheet ws = root_.getActiveWorksheet();
                        logger.debug("MouseButton3");
                        if (ws != null && isTryMatchAllowed()) {
                            popup_menu_.show(e.getComponent(), e.getX(), e
                                    .getY());
                        }
                        view_tree_.setSelectionPath(sel_path_);
                    }
                }
            }
        };
        view_tree_.addMouseListener(ml_);
    }

    // creates the id for the calc_head_ from the selected path in the tree
    private KEStoreID createCalcHeadCompoundIDFromPath() {
        KEStoreID chcmpID = new KEStoreID();
        Object[] path = sel_path_.getPath();
        for (int i = path.length - 1; i > 0; i--) {
            chcmpID.addString(((ProblemNode) path[i]).getId());
        }
        return chcmpID;
    }

    private boolean isTryMatchAllowed() {
        //    Worksheet ws = root.getActiveWorksheet();
        //    return ws == null ? false : ws.isCallProblemAllowed();
        //FIXME: remove!!! only a hack to show try_match_/try_refine_
        // functionality
        return true;
    }

    private void doTryMatch(KEStoreID chcmpID) {
        Worksheet ws = root_.getActiveWorksheet();
        ws.tryMatch(chcmpID);
    }

    private void doTryRefine(KEStoreID chcmpID) {
        Worksheet ws = root_.getActiveWorksheet();
        ws.tryRefine(chcmpID);
    }

    public void actionPerformed(ActionEvent e) {
        KEStoreID chcmpID = createCalcHeadCompoundIDFromPath();
        if (e.getSource() == try_refine_) {
            KEStoreID chID = createCalcHeadCompoundIDFromPath();
            doTryRefine(chcmpID);
        } else if (e.getSource() == try_match_) {
            KEStoreID chID = createCalcHeadCompoundIDFromPath();
            doTryMatch(chcmpID);
        }
    }
}